function [A] = diag_ls(A,rrank)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% input:
% A : Symmetric integer matrix with main diagonal set to zero
% rrank : Effective rank of the original matrix. If unknown, then set to 0.
%
% output:
% A : Returns the same as the input matrix, but with integer (non-zero) diagonal entries
%
% description:
% This algorithm uses non-negative least squares method on submatrices of A to estimate the missing values in the main diagonal of A.
% It is assumed that the true A has low (effective) rank.
%
% Written by Dora Erdos, Boston University, 2013
%
%



[n,~] = size(A)
if rrank == 0 | rrank>floor(n/2)
  	rrank = floor(n/2);
end

%r is the size of the submatrix we use
%it is more than the effective rank, so that the chosen
%submatrix is rrank with high probability
r = rrank * round(log(rrank));
r = min(r,floor(n/4));

%estimate main diagonal element unsing random submatrix not containing main diag element
for i=1:n
	%a submatrix of size rxr is chosen by choosing its row and column indices unif at random
	indices = randperm(n,2*r);
	c = find(indices == i);
	%make sure that row or column i are not part of the submatrix
	while size(c)>0
		indices(c)= randint(1,1,[1,n]);
		c = find(indices == i);
	end
	cols = indices(1:r);
  	rows = indices(r+1:2*r);
	W = A(i,cols);
	X = A(rows,cols);
	Y = A(rows,i);
	b = lsqnonneg(X,Y);
	t = W*b;
	A(i,i) = t;
end



% round the values in the main diagonal of A to the nearest integer
A = round(A);

